package PureMessage::HTTPD::Services::Application;
use strict;
use warnings;
use base qw( CGI::Application );
use PureMessage::Version qw($PREFIX);
use PureMessage::_sys;
use File::Path qw(mkpath);
use File::Basename qw(dirname);

# CGI::Application "init" hook, which sets up our defaults.
sub cgiapp_init {
    my $self = shift;

    # Apache sets STDIN = STDOUT = fd1. This creates problems when 
    # trying to dup STDIN and STDOUT to different fd's (ie to create
    # shell pipelines). So we reset STDIN to be fd0.
    untie(*STDIN);

    # default logfile
    $self->{__PM_LOGFILE} = 'services_log';
}

# sets the name of the logfile.
sub set_logfile {
    my ($self, $logfile) = @_;
    $self->{__PM_LOGFILE} = $logfile;
}

# logs message(s) to our logfile.
sub log {
    my ($self, @msgs) = @_;
    my $logfile = $self->{__PM_LOGFILE};
    $logfile = "$PREFIX/var/log/$logfile";
    _make_logfile( $logfile ) unless -e ($logfile);
    my $now = localtime;
    foreach my $msg (@msgs) {
        chomp( $msg );
        $msg = "[$now] $msg\n";
        PureMessage::_sys::append( $logfile, $msg );
    }
}

# creates the path for the logfile if it doesn't exist yet.
sub _make_logfile {
    my $dir = dirname(shift);
    unless (-d $dir) {
        mkpath $dir or die "Can't mkpath $dir: $!";
    }
}

1;

=head1 NAME

PureMessage::HTTPD::Services::Application - Base application class for HTTP services

=head1 SYNOPSIS

  package PureMessage::HTTPD::Services::MyApp;

  # derive from base application class
  use base qw( PureMessage::HTTPD::Services::Application );

  # setup; specify logfile and list of run modes
  sub setup {
      $self->set_logfile( 'my_log' );
      $self->run_modes(
        ...
        );
  }

=head1 DESCRIPTION

C<PureMessage::HTTPD::Services::Application> is a base application class to be
used by all HTTP services.  C<PureMessage::HTTPD::Services::Application> is
derived from C<CGI::Application>, and any/all methods callable on a
C<CGI::Application> object are available.

=head1 METHODS

=over

=item B<set_logfile($filename)>

Sets the name of the file to be used for logging by the service.  This method
should be called from your own over-ridden C<setup()> method, to set the name
of the logfile that you wish to use.  By default, F<services_log> is used
unless this method is called.

Please note that you only need to call this method with the I<name> of the
logfile that you wish to use; it will automatically be placed in the
F<E<lt>pmxE<gt>/var/log/> directory for you.

=item B<log(@messages)>

Logs one or more messages to the logfile specified by C<set_logfile()>.

=back

=head1 COPYRIGHT

Copyright (C) 2000-2006 Sophos Group.  All rights reserved.  Sophos and
PureMessage are trademarks of Sophos Plc and Sophos Group.

=cut
