package PureMessage::HTTPD::Services::Dispatch;

use strict;
use warnings;
use Apache::Constants qw(OK DECLINED);

# Mod_perl handler, to invoke the HTTPD service
sub handler {
    my $r = shift;

    # get our PATH_INFO; that explains which module implements the service
    my $path_info = $r->path_info();
    if (!$path_info || $path_info eq '/') {
        $r->warn( 'no PATH_INFO given to service dispatcher' );
        return DECLINED;
    }

    # get the name of the module implementing the service from the PATH_INFO,
    # and load up that module.
    my $module = get_module_name( $path_info );
    eval "require $module";
    if ($@) {
        $r->log_error( $@ );
        return DECLINED;
    }

    # instantiate the module implementing the service, and tell it to get the
    # run_mode out of the second component in the PATH_INFO
    my $app = $module->new(
                PARAMS => {
                    r => $r,                # Apache request object
                } );
    $app->mode_param( path_info=>2 );

    # run the application
    $app->run();
    return OK;
}

# Returns the name of the Perl module that implements the service requested
# in the given PATH_INFO.
sub get_module_name {
    my $path_info = shift;

    # get the stuff between the first+second '/' characters
    my $partial_path = (split(/\//, $path_info))[1];

    # turn it into a module name
    #   - all modules live under "PM::HTTPD::Services::"
    #   - module name parts have their first letter UC'd
    #   - '_'s turn into '::'s
    # e.g. "foo_bar" becomes "PM::HTTPD::Services::Foo::Bar"
    my $module = 'PureMessage::HTTPD::Services::'
                 . join( '::', (map { ucfirst } (split(/_/, $partial_path ))) );
    ($module) = ($module =~ /^([A-Za-z][A-Za-z0-9_\-\:]+)$/);
    if (!$module) {
        die "invalid characters used in module name";
    }

    # return the module name back to the caller
    return $module;
}

1;

=head1 NAME

PureMessage::HTTPD::Services::Dispatch - Dispatch PureMessage HTTPD Services

=head1 SYNOPSIS

  # in your Apache/Mod_perl configuration
  PerlModule PureMessage::HTTPD::Services::Dispatch
  <Location /services>
    SetHandler   perl-script
    PerlHandler  PureMessage::HTTPD::Services::Dispatch
  </Location>

=head1 DESCRIPTION

C<PureMessage::HTTPD::Services::Dispatch> provides a Mod_perl handler that
dispatches PureMessage HTTPD Services.  It looks at the C<PATH_INFO> in the
HTTP request and from that determines the name of the module implementing the
service and the run mode to dispatch in that module.

This module is based in part on the design of C<CGI::Application::Dispatch>,
but has been significantly trimmed down to only include the functionality that
we require.  C<CGI::Application::Dispatch> also required a bunch of other
modules to be installed during the build/test cycle which aren't part of our
standard build (and are likely not going to be any time soon).  Thus, the
functionality required was extracted out to this module.

=head1 PATH_INFO PARSING

When presented with C<PATH_INFO>, this module uses that information to help
determine which module implements the service requested, and the run mode to
invoke on that service.

=head2 Getting the module name

To retrieve the module name, C<PureMessage::HTTPD::Services::Dispatch> splits
the provided C<PATH_INFO> on slashes, and takes the first component in the
results and uses that to create the name of the module.

This component is then transformed so that understores are turned into
double-colons, and each word is passed through C<ucfirst()> to capitalize the
first letter of each word.  All services are then deemed to live underneath the
C<PureMessage::HTTPD::Services::*> namespace.

If no run mode is parsable from the C<PATH_INFO>, none is invoked on the
service and you can rely on the default C<CGI::Application> behaviour to do
what you need.

=head2 Getting the run mode

Similarly to how the module name is extracted from the C<PATH_INFO>, so is the
name of the run mode to invoke on the service.  Instead of getting the run mode
from the first component in the C<PATH_INFO>, though, it comes from the second
component.  No transformation or modification is done to this value.

=head2 Examples

  /services/module/start
    is implemented by "PureMessage::HTTPD::Services::Module"
    and has a run mode of "start"

  /services/foo_bar/list
    is implemented by "PureMessage::HTTPD::Services::Foo::Bar"
    and has a run mode of "list"

  /services/myapp
    is implemented by "PureMessage::HTTPD::Services::Myapp"
    and uses the default CGI::Application run mode

=head1 COPYRIGHT

Copyright (C) 2000-2006 Sophos Group.  All rights reserved.  Sophos and
PureMessage are trademarks of Sophos Plc and Sophos Group.

=cut
